from django.conf import settings
from django.contrib.contenttypes.models import ContentType

from creme import persons
from creme.creme_core.gui.bricks import Brick
from creme.creme_core.management.commands.creme_populate import BasePopulator
from creme.creme_core.models import BrickDetailviewLocation, ButtonMenuItem
from creme.geolocation.bricks import (
    OpenStreetMapDetailMapBrick,
    OpenStreetMapNeighboursMapBrick,
)

LEFT = BrickDetailviewLocation.LEFT
RIGHT = BrickDetailviewLocation.RIGHT
BOTTOM = BrickDetailviewLocation.BOTTOM


class Populator(BasePopulator):
    dependencies = ['creme_core', 'persons', 'pilote_extended']
    overwrite = settings.POPULATE_OVERWRITE

    def __init__(self, *args, **kwargs):
        self.Organisation = persons.get_organisation_model()
        self.Contact = persons.get_contact_model()
        return super().__init__(*args, **kwargs)

    def populate(self):
        """
        This module is only intended to delete (bleach)
        some already provided data
        and, in the case of display, to re-order some brick
        so far, there is not really a notion of 'already_populated'
        then we only rely on overwrite
        """
        if not self.overwrite:
            return

        org_ct_id = ContentType.objects.get_for_model(self.Organisation).id
        cnt_ct_id = ContentType.objects.get_for_model(self.Contact).id

        btn_todel = [
            'button_persons-become_customer',
            'button_persons-become_prospect',
            'button_persons-become_suspect',
            'button_persons-become_inactive',
            'button_persons-become_supplier',
            'button_persons-add_linked_contact',
            'button_opportunities-linked_opportunity',
            'button_tickets-linked2ticket',
        ]
        ButtonMenuItem.objects.filter(
            content_type_id=org_ct_id,
            button_id__in=btn_todel,
        ).delete()

        # replace google map by openstreetmap
        # delete google map detail view
        gmaps = [
            Brick.generate_id('geolocation', e)
            for e in ('detail_google_maps', 'google_whoisaround')
        ]
        org_todel = gmaps + [
            'block_persons-managers',
            'block_persons-employees',
            'block_creme_core-relations',
            'block_assistants-todos',
            'block_assistants-memos',
            'block_assistants-messages',
            'block_documents-linked_docs',
            'block_creme_core-history',
            'block_creme_core-properties',
            'block_persons-addresses_pretty',
            'block_persons-other_addresses_pretty',
            'block_activities-past_activities',
            'block_activities-future_activities',
            OpenStreetMapDetailMapBrick.id_,
            OpenStreetMapNeighboursMapBrick.id_,
            'specificblock_creme_config-persons-subject_customer_supplier',
            'specificblock_creme_config-persons-object_customer_supplier',
            'block_emails-mails_history',
            'block_opportunities-target_organisations',
            'block_assistants-alerts',
        ]
        cnt_todel = gmaps
        BrickDetailviewLocation.objects.filter(
            content_type_id=org_ct_id, brick_id__in=org_todel
        ).delete()
        BrickDetailviewLocation.objects.filter(
            content_type_id=cnt_ct_id, brick_id__in=cnt_todel
        ).delete()

        BrickDetailviewLocation.objects.multi_create(
            defaults={'model': self.Organisation, 'zone': LEFT},
            data=[
                {'brick': 'block_creme_core-relations', 'order': 9},
                {'brick': 'block_assistants-todos', 'order': 12},
                {'brick': 'block_assistants-memos', 'order': 13},
                {'brick': 'block_assistants-messages', 'order': 14},
                {'brick': 'block_documents-linked_docs', 'order': 15},
                {'brick': 'block_creme_core-history', 'order': 16},
            ],
        )
        BrickDetailviewLocation.objects.multi_create(
            defaults={'model': self.Organisation, 'zone': RIGHT},
            data=[
                {'brick': 'block_persons-addresses_pretty', 'order': 5},
                {'brick': 'block_persons-other_addresses_pretty', 'order': 6},
                {'brick': OpenStreetMapDetailMapBrick, 'order': 7},
                {'brick': 'block_activities-past_activities', 'order': 8},
                {'brick': 'block_activities-future_activities', 'order': 9},
            ],
        )

        for model in (self.Organisation, self.Contact):
            BrickDetailviewLocation.objects.multi_create(
                defaults={'model': model, 'zone': BOTTOM},
                data=[
                    {'brick': OpenStreetMapNeighboursMapBrick, 'order': 600},
                ],
            )
