from django.utils.translation import gettext as _

from creme.creme_core.auth.entity_credentials import EntityCredentials
from creme.creme_core.bricks import Relation, RelationsBrick
from creme.creme_core.gui.bricks import SimpleBrick
from creme.creme_core.models import RelationType
from creme.persons.bricks import (
    ContactCardHatBrick,
    OrganisationCardHatBrick,
    QuerysetBrick,
)

from .utils import rtype_ids_from_sym


class MyOrganisationCardHatBrick(OrganisationCardHatBrick):
    id_ = SimpleBrick._generate_hat_id('pilote-extended', 'organisation_card')
    verbose_name = "Chapeau organisation"
    template_name = 'persons/bricks/my-organisation-hat-card.html'

    def detailview_display(self, context):
        user = context['user']
        organisation = context['object']
        context.update(
            {
                'electees': EntityCredentials.filter(
                    user, organisation.get_electees()
                )[:16],
            }
        )
        return super().detailview_display(context)


class MyContactCardHatBrick(ContactCardHatBrick):
    id_ = SimpleBrick._generate_hat_id('pilote-extended', 'contact_card')
    verbose_name = "Chapeau contact"
    template_name = 'persons/bricks/my-contact-hat-card.html'

    def detailview_display(self, context):
        user = context['user']
        contact = context['object']
        context.update(
            {
                'managed_org': EntityCredentials.filter(
                    user, contact.get_managed_org()
                )[:16],
                'employed_by': EntityCredentials.filter(
                    user, contact.get_employed_by()
                )[:16],
                'elected_of': EntityCredentials.filter(
                    user, contact.get_elected_of()
                )[:16],
            }
        )
        return super().detailview_display(context)


class LimitedRelationsBrick(RelationsBrick):
    def detailview_display(self, context):
        """I wish a hook exists to not repeat most of the code of
        creme_core/bricks.py:RelationsBrick.detailview_display"""
        entity = context['object']
        relations = entity.relations.select_related(
            'type',
            'type__symmetric_type',
            'object_entity',
        )
        included_rtype_ids = self._included_rtype_ids
        excluded_rtype_ids = self._excluded_rtype_ids
        included_rtype_sym = getattr(self, 'relation_type_deps_sym', None)
        reloading_info = self._reloading_info

        # NB: it's not a reload, it's the initial render()
        if reloading_info is None:
            # simpl. of BricksManager.get(context).used_relationtype_ids
            used_rtype_ids = self.relation_type_deps

            excluded_rtype_ids_set = {
                *RelationType.objects.filter(
                    id__in=used_rtype_ids,
                    minimal_display=True,
                ).values_list('id', flat=True),
            }
            included_rtype_ids.extend(
                rtype_id
                for rtype_id in used_rtype_ids
                if rtype_id not in excluded_rtype_ids_set
            )
            excluded_rtype_ids.extend(excluded_rtype_ids_set)

            self._reloading_info = reloading_info = {}
            if included_rtype_ids:
                reloading_info['include'] = included_rtype_ids
            if excluded_rtype_ids:
                reloading_info['exclude'] = excluded_rtype_ids
            if included_rtype_sym:
                reloading_info['include_sym'] = included_rtype_sym
        else:
            get = reloading_info.get
            included_rtype_ids.extend(get('include', ()))
            excluded_rtype_ids.extend(get('exclude', ()))
            included_rtype_sym = get('include_sym', ())

        if excluded_rtype_ids:
            relations = relations.exclude(type__in=excluded_rtype_ids)
        all_included_rtype_ids = included_rtype_ids.copy()
        if included_rtype_sym:
            all_included_rtype_ids.extend(
                rtype_ids_from_sym(included_rtype_sym)
            )
        if all_included_rtype_ids:
            relations = relations.filter(type__in=all_included_rtype_ids)

        btc = self.get_template_context(
            context,
            relations,
            excluded_rtype_ids=excluded_rtype_ids,
            included_rtype_ids=included_rtype_ids,
            included_rtype_sym=included_rtype_sym,
        )
        # NB: DB optimisation
        Relation.populate_real_object_entities(btc['page'].object_list)
        return self._render(btc)


class MyManagersBrick(LimitedRelationsBrick):
    id_ = QuerysetBrick.generate_id('pilote-extended', 'managers')
    verbose_name = _('Organisation managers')
    template_name = 'persons/bricks/my-managers.html'
    relation_type_deps_sym = 'ROLE_OBJ_RESPONSABLES'


class MyEmployeesBrick(LimitedRelationsBrick):
    id_ = QuerysetBrick.generate_id('pilote-extended', 'employees')
    verbose_name = _('Organisation employees')
    template_name = 'persons/bricks/my-employees.html'
    relation_type_deps_sym = 'ROLE_OBJ_SALARIE'


bricks_list = (MyManagersBrick, MyEmployeesBrick)
