# -*- coding: utf-8 -*-
import datetime
import logging
from functools import cache

from django.db.models.query_utils import Q

from creme.creme_core.core.function_field import (
    FunctionField,
    FunctionFieldLink,
    FunctionFieldResultsList,
)
from creme.creme_core.forms import listview as lv_form
from creme.creme_core.models import CremeEntity, Relation

from center_pilote.info_tables.models import TypeAdhesion

from . import constants
from .utils import compress_interval

logger = logging.getLogger(__name__)


@cache
def type_adh_choices():
    return [
        {'value': id, 'label': name}
        for id, name in TypeAdhesion.objects.values_list('id', 'name')
    ]


class IsAdherentSearchField(lv_form.ListViewSearchField):
    widget = lv_form.BooleanLVSWidget

    def to_python(self, value):
        if value is None:
            return Q()
        annee = datetime.date.today().year
        annees = [annee - 1, annee]
        ret_q = Q(info_tables_adhesions__annee__range=annees)
        if value:
            return ret_q
        return ~ret_q


class IsAdherentField(FunctionField):
    name = 'org-adherente'
    verbose_name = "Est adhérent (N, N-1)"
    search_field_builder = IsAdherentSearchField

    def __call__(self, entity, user):
        return self.result_type(
            "oui" if entity.est_adherent_annee() else "non"
        )


class AnneeAdhesionSearchField(lv_form.RegularPositiveIntegerField):
    def _build_q_from_operations(self, operations):
        return Q(
            **{
                f'info_tables_adhesions__annee__{op}': number
                for op, number in operations
            }
        )


class AnneeAdhesionField(FunctionField):
    name = 'org-annee-adhesion'
    verbose_name = "Années d'adhésion"
    search_field_builder = AnneeAdhesionSearchField

    def __call__(self, entity, user):
        adh_qs = entity.info_tables_adhesions.order_by('annee')
        year_list = compress_interval(adh_qs.values_list('annee', flat=True))
        return self.result_type(year_list)


class TypeAdhesionSearchField(lv_form.BaseChoiceField):
    def _build_choices(self):
        choices = super()._build_choices(null_label=None)
        return choices + type_adh_choices()

    def _get_q_for_choice(self, choice_value):
        return Q(info_tables_adhesions__type_adhesion__id=choice_value)


class TypeAdhesionField(FunctionField):
    name = 'org-type-adhesion'
    verbose_name = "Type(s) d'adhésion"
    search_field_builder = TypeAdhesionSearchField

    def __call__(self, entity, user):
        adh_qs = entity.info_tables_adhesions
        adh_qs = adh_qs.order_by('annee')
        adh_qs = adh_qs.select_related('type_adhesion')
        value = ', '.join(
            f"{annee}-{type_adh}"
            for annee, type_adh in adh_qs.values_list(
                'annee', 'type_adhesion__name'
            )
        )
        return self.result_type(value)


class RelIsSymbolSearchField(lv_form.ListViewSearchField):
    widget = lv_form.TextLVSWidget
    rtype_sym = None

    def to_python(self, value):
        if not value:
            return Q()
        ret_q = Q(
            pk__in=Relation.objects.filter(
                type__in=self.rtype_sym,
                object_entity__header_filter_search_field__icontains=value,
            ).values_list('subject_entity', flat=True)
        )
        return ret_q


class RelIsManagerSearchField(RelIsSymbolSearchField):
    rtype_sym = constants.ROLE_OBJ_RESPONSABLES


class RelIsEmployeeSearchField(RelIsSymbolSearchField):
    rtype_sym = constants.ROLE_OBJ_SALARIE


class RelManageSearchField(RelIsSymbolSearchField):
    rtype_sym = constants.ROLE_SUB_RESPONSABLES


class RelIsEmployedBySearchField(RelIsSymbolSearchField):
    rtype_sym = constants.ROLE_SUB_SALARIE


class RelIsSymbolField(FunctionField):
    "place to factorize code"
    rtype_sym = None

    def __call__(self, entity, user):
        rel_qs = entity.relations.filter(type_id__in=self.rtype_sym)
        rel_ids = rel_qs.values_list('object_entity_id', flat=True)
        entity_qs = CremeEntity.objects.filter(id__in=rel_ids)
        ff_link_list = [
            FunctionFieldLink(str(elm), elm.get_absolute_url())
            for elm in entity_qs
        ]
        return FunctionFieldResultsList(ff_link_list)


class RelIsManagerField(RelIsSymbolField):
    "on affiche toutes les relations de type ROLE_OBJ_RESPONSABLES"
    name = 'org-relations-managers'
    verbose_name = "A pour responsable(s) (toutes catégories)"
    rtype_sym = constants.ROLE_OBJ_RESPONSABLES
    search_field_builder = RelIsManagerSearchField


class RelIsEmployeeField(RelIsSymbolField):
    "on affiche toutes les relations de type ROLE_OBJ_SALARIE"
    name = 'org-relations-employees'
    verbose_name = "A pour salarié(s) (toutes catégories)"
    rtype_sym = constants.ROLE_OBJ_SALARIE
    search_field_builder = RelIsEmployeeSearchField


class RelManageField(RelIsSymbolField):
    "on affiche toutes les relations de type ROLE_SUB_RESPONSABLES"
    name = 'contact-relations-managers'
    verbose_name = "Responsable de (toutes catégories)"
    rtype_sym = constants.ROLE_SUB_RESPONSABLES
    search_field_builder = RelManageSearchField


class RelIsEmployedByField(RelIsSymbolField):
    "on affiche toutes les relations de type ROLE_SUB_SALARIE"
    name = 'contact-relations-employees'
    verbose_name = "Est salarié(e) de (toutes catégories)"
    rtype_sym = constants.ROLE_SUB_SALARIE
    search_field_builder = RelIsEmployedBySearchField
