# -*- coding: utf-8 -*-
import datetime

from django.db.models import (
    BooleanField,
    CharField,
    ForeignKey,
    Index,
    Manager,
    PositiveIntegerField,
    QuerySet,
)

from creme.creme_core import models as creme_models
from creme.persons import get_organisation_model
from creme.persons.models.contact import AbstractContact

from ..constants import (
    ROLE_EMPLOYERS,
    ROLE_OBJ_BLOC_RELATION,
    ROLE_OBJ_RESPONSABLES,
    ROLE_OBJ_SALARIE,
)
from ..utils import SlugModelMixin, SlugQuerySetMixin, join_slugify


class ContactQuerySet(SlugQuerySetMixin, QuerySet):
    pass


class ContactManager(Manager):
    pass


class Contact(SlugModelMixin, AbstractContact):
    class Meta(AbstractContact.Meta):
        app_label = 'pilote_extended'
        indexes = [
            Index(fields=['slug_id']),
        ]

    objects = ContactManager.from_queryset(ContactQuerySet)()
    import_id = PositiveIntegerField(
        "ID temporaire d'import",
        blank=True,
        null=True,
        db_index=True,
    )
    slug_id = CharField(
        "ID de dedoublonnage", max_length=1024, default='', blank=True
    ).set_tags(optional=True)
    emailing = BooleanField("Emailing", null=True, blank=True).set_tags(
        optional=True
    )
    representant_de = ForeignKey(
        'pilote_extended.Organisation',
        verbose_name="Représentant (fibois) de",
        on_delete=creme_models.CREME_REPLACE,
        null=True,
        blank=True,
    ).set_tags(optional=True)
    membre_bureau = BooleanField(
        "Membre du bureau", null=True, blank=True
    ).set_tags(optional=True)
    fonction_bureau = ForeignKey(
        'info_tables.FonctionBureau',
        verbose_name="Fonction dans le bureau",
        on_delete=creme_models.CREME_REPLACE,
        null=True,
        blank=True,
    ).set_tags(optional=True)
    membre_ca = BooleanField("Membre du CA", null=True, blank=True).set_tags(
        optional=True
    )
    membre_ag = BooleanField("Membre de l'AG", null=True, blank=True).set_tags(
        optional=True
    )
    invite_ag = BooleanField("Invité à l'AG", null=True, blank=True).set_tags(
        optional=True
    )

    def __str__(self):
        ret = super().__str__()
        if self.est_adherent_annee():
            return f"{ret} *"
        return ret

    def get_employers(self):
        return get_organisation_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_EMPLOYERS,
            relations__object_entity=self.id,
        )

    def get_employed_by(self):
        return get_organisation_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_OBJ_SALARIE,
            relations__object_entity=self.id,
        )

    def get_managed_org(self):
        return get_organisation_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_OBJ_RESPONSABLES,
            relations__object_entity=self.id,
        )

    def get_elected_of(self):
        return get_organisation_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_OBJ_BLOC_RELATION,
            relations__object_entity=self.id,
        )

    def est_adherent_annee(self, annee=None):
        if annee is None:
            annee = datetime.date.today().year
        annees = [annee - 1, annee]
        return self.info_tables_adhesions.filter(annee__range=annees).exists()

    def slug(self):
        return join_slugify(self.first_name, self.last_name)
