# -*- coding: utf-8 -*-
import datetime

from django.conf import settings
from django.db.models import (
    BooleanField,
    CharField,
    DecimalField,
    EmailField,
    ForeignKey,
    Index,
    ManyToManyField,
    PositiveIntegerField,
    QuerySet,
)

from creme.creme_core import models as creme_models
from creme.persons import get_contact_model
from creme.persons.models.organisation import (
    AbstractOrganisation,
    OrganisationManager,
)

from ..constants import (
    ROLE_SUB_BLOC_RELATION,
    ROLE_SUB_RESPONSABLES,
    ROLE_SUB_SALARIE,
)
from ..utils import SlugModelMixin, SlugQuerySetMixin, join_slugify
from .fields import phone_repr


class OrganisationQuerySet(SlugQuerySetMixin, QuerySet):
    @staticmethod
    def _as_dict_for_directory(inst):
        return {
            'id': inst.id,
            'name': inst.name,
            'email': inst.email,
            'siret': inst.siret,
            'naf': inst.naf,
            'phone': inst.get_phone_display(),
            'adherent': inst.est_adherent_annee(),
            'city': getattr(inst.billing_address, 'city', ''),
            'address': getattr(inst.billing_address, 'address', ''),
            'zipcode': getattr(inst.billing_address, 'zipcode', ''),
            'latitude': getattr(
                getattr(inst.billing_address, 'geoaddress', None),
                'latitude',
                None,
            ),
            'longitude': getattr(
                getattr(inst.billing_address, 'geoaddress', None),
                'longitude',
                None,
            ),
            'geo_status': getattr(
                getattr(inst.billing_address, 'geoaddress', None),
                'status',
                None,
            ),
            # adresse (ville, adresse, lat. lon),
            'website': inst.url_site,
            'activite_pri': inst.activite_id,
            'activite_sec': [elm.id for elm in inst.activite_secondaire.all()],
        }

    def directory_as_list_of_dict(self):
        query = (
            self.select_related(
                'billing_address',
                'billing_address__geoaddress',
                'activite',
            )
            .prefetch_related('activite_secondaire')
            .filter(annuaire_portail=True)
        )
        return list(map(self._as_dict_for_directory, query))


class Organisation(SlugModelMixin, AbstractOrganisation):
    class Meta(AbstractOrganisation.Meta):
        app_label = 'pilote_extended'
        indexes = [
            Index(fields=['slug_id']),
        ]

    objects = OrganisationManager.from_queryset(OrganisationQuerySet)()

    import_id = PositiveIntegerField(
        "ID temporaire d'import",
        blank=True,
        null=True,
        db_index=True,
    )
    slug_id = CharField(
        "ID de dedoublonnage", max_length=1024, default='', blank=True
    ).set_tags(optional=True)
    activite = ForeignKey(
        'info_tables.Activite',
        verbose_name="Activité principale",
        blank=True,
        null=True,
        on_delete=creme_models.CREME_REPLACE,
    ).set_tags(optional=True)

    activite_secondaire = ManyToManyField(
        'info_tables.Activite',
        verbose_name="Activité(s) secondaire(s)",
        related_name='organisation_act_sec_set',
        blank=True,
    ).set_tags(optional=True)

    contact_adherent = ForeignKey(
        settings.PERSONS_CONTACT_MODEL,
        verbose_name="Contact adhérent",
        blank=True,
        null=True,
        on_delete=creme_models.CREME_REPLACE,
    ).set_tags(optional=True)

    activite_sous_traite = ManyToManyField(
        'info_tables.Activite',
        verbose_name="Activité sous traitée",
        related_name='organisation_act_ss_trait_set',
        blank=True,
    ).set_tags(optional=True)

    secteur_intervention = ManyToManyField(
        'info_tables.SecteurIntervention',
        verbose_name="Secteur d'intervention",
        blank=True,
    ).set_tags(optional=True)

    marquage_ce = ManyToManyField(
        'info_tables.MarquageCe',
        verbose_name="Marquage CE",
        blank=True,
    ).set_tags(optional=True)

    type_client = ManyToManyField(
        'info_tables.TypeClient',
        verbose_name="Type de client",
        blank=True,
    ).set_tags(optional=True)

    essence_majoritaire = ForeignKey(
        'info_tables.EssenceArbre',
        verbose_name="Essence majoritaire",
        on_delete=creme_models.CREME_REPLACE,
        related_name='org_ess_maj',
        null=True,
        blank=True,
    ).set_tags(optional=True)

    equipements = ManyToManyField(
        'info_tables.Equipement',
        verbose_name="Équipements",
        blank=True,
    ).set_tags(optional=True)

    feuillus = ManyToManyField(
        'info_tables.EssenceArbre',
        verbose_name="Feuillus",
        related_name='org_ess_feui',
        limit_choices_to={'type_ess': 'feui'},
        blank=True,
    ).set_tags(optional=True)

    resineux = ManyToManyField(
        'info_tables.EssenceArbre',
        verbose_name="Résineux",
        related_name='org_ess_resi',
        limit_choices_to={'type_ess': 'resi'},
        blank=True,
    ).set_tags(optional=True)

    amenagement_paysager = ManyToManyField(
        'info_tables.AmenagementPaysager',
        verbose_name="Aménagements paysagers",
        blank=True,
    ).set_tags(optional=True)

    bois_nrj_connexes = ManyToManyField(
        'info_tables.BoisNrjConnexe',
        verbose_name="Bois énergie connexe",
        blank=True,
    ).set_tags(optional=True)

    construction = ManyToManyField(
        'info_tables.Construction',
        verbose_name="Construction",
        blank=True,
    ).set_tags(optional=True)

    emballages = ManyToManyField(
        'info_tables.Emballage',
        verbose_name="Emballage",
        blank=True,
    ).set_tags(optional=True)

    menuiseries_revetements = ManyToManyField(
        'info_tables.MenuiserieRevetement',
        verbose_name="Menuiseries revetements",
        blank=True,
    ).set_tags(optional=True)

    traitements_finitions = ManyToManyField(
        'info_tables.TraitementFinition',
        verbose_name="Traitements finitions",
        blank=True,
    ).set_tags(optional=True)
    entreprise_radiee = BooleanField(
        "Entreprise radiée", null=True, blank=True
    ).set_tags(optional=True)
    phone_cell = creme_models.fields.PhoneField(
        "Mobile", max_length=100, blank=True
    ).set_tags(optional=True)
    email_2 = EmailField("Email secondaire", blank=True).set_tags(
        optional=True
    )
    emailing = BooleanField("Emailing", null=True, blank=True).set_tags(
        optional=True
    )
    annuaire_portail = BooleanField(
        "Annuaire portail", null=True, blank=True
    ).set_tags(optional=True)
    charte_bois_local = BooleanField(
        "Charte bois local", null=True, blank=True
    ).set_tags(optional=True)
    bois_local_trans = PositiveIntegerField(
        "Bois local transformé %", null=True, blank=True
    ).set_tags(optional=True)
    volume_grume_annuel = PositiveIntegerField(
        "Volume grume annuel (m3)", null=True, blank=True
    ).set_tags(optional=True)
    pefc = BooleanField("PEFC", null=True, blank=True).set_tags(optional=True)
    fsc = BooleanField("FSC", null=True, blank=True).set_tags(optional=True)
    marquage_ctb = CharField(
        "marquage CTB", blank=True, max_length=64
    ).set_tags(optional=True)
    nimp15 = BooleanField("NIMP15", null=True, blank=True).set_tags(
        optional=True
    )
    etf_gestion_durable = BooleanField(
        "ETF gestion durable", null=True, blank=True
    ).set_tags(optional=True)
    agrement_phyto = BooleanField(
        "Agrément phytosanitaire", null=True, blank=True
    ).set_tags(optional=True)
    lng_sciage_max = DecimalField(
        "Longueur de sciage maximum (m)",
        decimal_places=2,
        max_digits=4,
        null=True,
        blank=True,
    ).set_tags(optional=True)

    def __str__(self):
        if self.est_adherent_annee():
            return f"{self.name} *"
        return self.name

    def get_phone_display(self):
        return phone_repr(self.phone)

    def est_adherent_annee(self, annee=None):
        if annee is None:
            annee = datetime.date.today().year
        annees = [annee - 1, annee]
        return self.info_tables_adhesions.filter(annee__range=annees).exists()

    def slug(self):
        city = self.billing_address.city if self.billing_address else ''
        return join_slugify(self.name, city, self.siret)

    def is_slug_up_to_date(self):
        slug_id = self.slug()
        if slug_id == self.slug_id:
            return True
        self.slug_id = slug_id
        return False

    def save(self, *args, **kwargs):
        "ajout de compute_slug=True/False"
        compute_slug = kwargs.pop('compute_slug', True)
        if compute_slug:
            self.is_slug_up_to_date()
        super().save(*args, **kwargs)

    def get_managers(self):
        return get_contact_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_SUB_RESPONSABLES,
            relations__object_entity=self.id,
        )

    def get_employees(self):
        return get_contact_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_SUB_SALARIE,
            relations__object_entity=self.id,
        )

    def get_electees(self):
        return get_contact_model().objects.filter(
            is_deleted=False,
            relations__type__in=ROLE_SUB_BLOC_RELATION,
            relations__object_entity=self.id,
        )
