from django.conf import settings

from creme import persons
from creme.activities.models import ActivityType
from creme.creme_core.gui.bricks import SimpleBrick
from creme.creme_core.management.commands.creme_populate import BasePopulator
from creme.creme_core.models import BrickDetailviewLocation, RelationType
from creme.creme_core.utils import create_if_needed

from .. import bricks, constants
from .brick import PopulateContactBricks, PopulateOrganisationBricks
from .form import PopulateForm
from .header_filter import PopulateOrgHeaderFilter
from .role import PopulateRole

HAT = BrickDetailviewLocation.HAT


class Populator(BasePopulator):
    dependencies = ['creme_core', 'persons', 'activities']

    def __init__(self, *args, **kwargs):
        self.overwrite = settings.POPULATE_OVERWRITE
        self.Organisation = persons.get_organisation_model()
        self.Contact = persons.get_contact_model()
        return super().__init__(*args, **kwargs)

    @staticmethod
    def _norm2dict(elm):
        if isinstance(elm, dict):
            return elm
        elif hasattr(elm, '_asdict'):
            return elm._asdict()

    def _dict_populate(self, modelname, constantname, MODELS):
        model_class = getattr(MODELS, modelname)
        already_populated = model_class.objects.exists()
        if already_populated:
            return
        CONSTANT = getattr(constants, constantname)
        model_class.objects.bulk_create(
            (model_class(**self._norm2dict(elm)) for elm in CONSTANT)
        )

    def _unpopulate_staff_size(self):
        model_class = persons.models.StaffSize
        if set(model_class.objects.values_list('size', flat=True)) == {
            '1 - 5',
            '100 - 500',
            '11 - 50',
            '51 - 100',
            '6 - 10',
            '> 500',
        }:
            model_class.objects.all().delete()

    def _unpopulate_legal_form(self):
        model_class = persons.models.LegalForm
        if set(model_class.objects.values_list('title', flat=True)) == {
            'SARL',
            'Association loi 1901',
            'SA',
            'SAS',
        }:
            model_class.objects.all().delete()

    def _populate_hat_brick(self):
        hat_brick_id = SimpleBrick._generate_hat_id(
            'pilote-extended', 'organisation_card'
        )
        if (
            not self.overwrite
            and BrickDetailviewLocation.objects.filter(
                brick_id=hat_brick_id
            ).exists()
        ):
            return
        # remove Organisation Hat brick
        BrickDetailviewLocation.objects.filter(
            content_type__model='organisation',
            zone=HAT,
        ).delete()
        # replace with our
        BrickDetailviewLocation.objects.create_if_needed(
            model=self.Organisation,
            zone=HAT,
            brick=bricks.MyOrganisationCardHatBrick,
            order=1,
        )
        # remove Contact Hat brick
        BrickDetailviewLocation.objects.filter(
            content_type__model='contact',
            zone=HAT,
        ).delete()
        # replace with our
        BrickDetailviewLocation.objects.create_if_needed(
            model=self.Contact,
            zone=HAT,
            brick=bricks.MyContactCardHatBrick,
            order=1,
        )

    def _populate_relation(self):
        if (
            not self.overwrite
            and RelationType.objects.filter(
                pk=constants.REL_SUB_PRESIDENT
            ).exists()
        ):
            return
        for rt_sub, rt_obj in getattr(constants, 'SIMPLE_RELATIONS_LIST', []):
            RelationType.create(
                rt_sub + ([self.Contact],),
                rt_obj + ([self.Organisation],),
                is_custom=True,
            )

    def _populate_activity_type(self):
        for pk, info in constants.POP_ACTIVITYTYPE.items():
            create_if_needed(
                ActivityType,
                {'pk': pk},
                is_custom=False,
                **info,
            )

    def populate(self):
        self._unpopulate_staff_size()
        self._unpopulate_legal_form()
        self._populate_relation()
        for modelname, constantname in constants.DICT_POPULATE_LIST:
            self._dict_populate(modelname, constantname, persons.models)
        PopulateForm().run()
        self._populate_hat_brick()
        PopulateContactBricks().run()
        PopulateOrganisationBricks().run()
        self._populate_activity_type()
        PopulateRole().run()
        PopulateOrgHeaderFilter().run()
