# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType

from creme.creme_core.core.entity_cell import EntityCellRegularField
from creme.creme_core.models import (
    BrickDetailviewLocation,
    CustomBrickConfigItem,
)
from creme.persons import get_contact_model, get_organisation_model

from center_pilote.populate_utils import Populate

from .. import bricks

LEFT = BrickDetailviewLocation.LEFT
RIGHT = BrickDetailviewLocation.RIGHT


class PopulateBricks(Populate):
    def _mk_cbci(self, id, name, fields):
        return CustomBrickConfigItem.objects.get_or_create(
            id=id,
            defaults={
                'name': name,
                'content_type': self.model,
                'cells': [
                    EntityCellRegularField.build(self.model, field)
                    for field in fields
                ],
            },
        )


class PopulateContactBricks(PopulateBricks):
    model = get_contact_model()

    def remove_existing(self):
        CustomBrickConfigItem.objects.filter(
            id__in=(
                'persons-contact_complementary',
                'persons-contact_adm_fibois',
            )
        ).delete()

    def is_already_populated(self):
        return CustomBrickConfigItem.objects.filter(
            id='persons-contact_complementary'
        ).exists()

    def populate(self):
        # remove complementary info
        BrickDetailviewLocation.objects.filter(
            brick_id__in=(
                'customblock-persons-contact_complementary',
                'customblock-persons-contact_adm_fibois',
                'customblock-persons-contact_identification',
            )
        ).delete()
        # replace with our
        cbci_contact_identification, created = self._mk_cbci(
            id='persons-contact_identification',
            name="Identification du contact",
            fields=('description',),
        )
        cbci_contact_adm_fibois, created = self._mk_cbci(
            id='persons-contact_adm-fibois',
            name="Administratif Fibois",
            fields=(
                'representant_de',
                'membre_bureau',
                'fonction_bureau',
                'membre_ca',
                'membre_ag',
                'invite_ag',
            ),
        )
        # create display
        BrickDetailviewLocation.objects.multi_create(
            defaults={'model': self.model, 'zone': LEFT},
            data=[
                {'brick': cbci_contact_identification.brick_id, 'order': 30},
                {'brick': cbci_contact_adm_fibois.brick_id, 'order': 40},
            ],
        )


class PopulateOrganisationBricks(PopulateBricks):
    model = get_organisation_model()

    def remove_existing(self):
        CustomBrickConfigItem.objects.filter(
            id__in=(
                'persons-organisation_identification',
                'persons-organisation_contact',
                'persons-organisation_adm_fibois',
                'persons-organisation_info_ent_bois',
                'persons-organisation_info_client',
                'persons-organisation_info_essences_trv',
                'persons-organisation_equipements',
                'persons-organisation_produits',
            )
        ).delete()

    def is_already_populated(self):
        return CustomBrickConfigItem.objects.filter(
            id='persons-organisation_identification'
        ).exists()

    def populate(self):
        # remove complementary info
        BrickDetailviewLocation.objects.filter(
            brick_id__in=(
                'customblock-persons-organisation_complementary',
                'block_pilote-extended-managers',
                'block_pilote-extended-employees',
            )
        ).delete()
        # replace with our
        cbci_orga_identification, created = self._mk_cbci(
            id='persons-organisation_identification',
            name="Identification de l'organisation",
            fields=(
                'name',
                'activite',
                'activite_secondaire',
                'staff_size',
                'creation_date',
                'naf',
                'siret',
                'siren',
                'legal_form',
                'annual_revenue',
                'image',
                'entreprise_radiee',
            ),
        )
        cbci_orga_contact, created = self._mk_cbci(
            id='persons-organisation_contact',
            name="Contacts de l'organisation",
            fields=(
                'phone',
                'phone_cell',
                'email',
                'email_2',
                'emailing',
                'url_site',
                'annuaire_portail',
                'fax',
            ),
        )
        cbci_orga_adm_fibois, created = self._mk_cbci(
            id='persons-organisation_adm_fibois',
            name="Administratif Fibois",
            fields=('contact_adherent',),
        )
        cbci_orga_info_ent_bois, created = self._mk_cbci(
            id='persons-organisation_info_ent_bois',
            name="Activité entreprise bois",
            fields=(
                'activite_sous_traite',
                'secteur_intervention',
                'charte_bois_local',
                'bois_local_trans',
                'volume_grume_annuel',
                'pefc',
                'fsc',
                'marquage_ce',
                'marquage_ctb',
                'nimp15',
                'etf_gestion_durable',
                'agrement_phyto',
            ),
        )
        cbci_orga_info_client, created = self._mk_cbci(
            id='persons-organisation_info_client',
            name="Info clients",
            fields=('type_client',),
        )
        cbci_orga_info_essences_trv, created = self._mk_cbci(
            id='persons-organisation_info_essences_trv',
            name="Info essences travaillées",
            fields=(
                'essence_majoritaire',
                'feuillus',
                'resineux',
            ),
        )
        cbci_orga_info_equipements, created = self._mk_cbci(
            id='persons-organisation_equipements',
            name="Info équipements",
            fields=('equipements',),
        )
        cbci_orga_info_produits, created = self._mk_cbci(
            id='persons-organisation_produits',
            name="Info produits",
            fields=(
                'amenagement_paysager',
                'bois_nrj_connexes',
                'construction',
                'emballages',
                'lng_sciage_max',
                'menuiseries_revetements',
                'traitements_finitions',
            ),
        )
        # remove our existing display related to this model
        model_ct = ContentType.objects.get_for_model(self.model).id
        BrickDetailviewLocation.objects.filter(
            content_type_id=model_ct,
            brick_id__in=(
                cbci_orga_contact.brick_id,
                cbci_orga_identification.brick_id,
                cbci_orga_adm_fibois.brick_id,
                bricks.MyManagersBrick.id_,
                bricks.MyEmployeesBrick.id_,
                cbci_orga_info_ent_bois.brick_id,
                cbci_orga_info_client.brick_id,
                cbci_orga_info_essences_trv.brick_id,
                cbci_orga_info_equipements.brick_id,
                cbci_orga_info_produits.brick_id,
            ),
        ).delete()
        # then re-create our display
        BrickDetailviewLocation.objects.multi_create(
            defaults={'model': self.model, 'zone': LEFT},
            data=[
                {'brick': cbci_orga_contact.brick_id, 'order': 5},
                {'brick': cbci_orga_identification.brick_id, 'order': 6},
                {'brick': bricks.MyManagersBrick, 'order': 7},
                {'brick': bricks.MyEmployeesBrick, 'order': 8},
                # block relation (see bleach.populate)
                # block adhesions (see info_tables.populate)
                {'brick': cbci_orga_adm_fibois.brick_id, 'order': 11},
                # block todos
                # block memos
                # block messages
                # block documents lies
                # block historique
            ],
        )
        BrickDetailviewLocation.objects.multi_create(
            defaults={'model': self.model, 'zone': RIGHT},
            data=[
                {'brick': cbci_orga_info_ent_bois.brick_id, 'order': 10},
                {'brick': cbci_orga_info_client.brick_id, 'order': 11},
                {'brick': cbci_orga_info_essences_trv.brick_id, 'order': 12},
                {'brick': cbci_orga_info_equipements.brick_id, 'order': 13},
                {'brick': cbci_orga_info_produits.brick_id, 'order': 14},
            ],
        )
