"""
Create a base role with:
- name: rolename
- allowed_apps: all
- administrable_apps: none
- permissions apply on: set_type
- permissions: values
- creatable_models: create_models_list
- exportable_models: create_models_list
"""
from django.contrib.contenttypes.models import ContentType
from django.db.models.deletion import ProtectedError

from creme.creme_core.apps import CremeAppConfig, creme_app_configs
from creme.creme_core.auth import EntityCredentials
from creme.creme_core.models import SetCredentials, UserRole
from creme.documents.models import Document, Folder
from creme.events.models import Event

from center_pilote.filiere_bois.models import InfraBoisNrj
from center_pilote.pilote_extended.models import (
    Activity,
    Contact,
    Organisation,
)
from center_pilote.populate_utils import Populate


class PopulateRole(Populate):
    rolename = 'base'
    set_type = SetCredentials.ESET_ALL
    value = (
        0
        | EntityCredentials.VIEW
        | EntityCredentials.CHANGE
        | EntityCredentials.DELETE
        | EntityCredentials.LINK
        | EntityCredentials.UNLINK
    )
    create_models_list = [
        Contact,
        Organisation,
        Activity,
        InfraBoisNrj,
        Event,
        Document,
        Folder,
    ]

    def remove_existing(self):
        """
        UserRole maybe already bound to user. In this case we do
        not really delete it (it raise a ProtectedError) but
        we want to continue the populate process
        """
        try:
            UserRole.objects.filter(name=self.rolename).delete()
        except ProtectedError:
            pass

    def is_already_populated(self):
        return UserRole.objects.filter(name=self.rolename).exists()

    def populate(self):
        self.role = UserRole.objects.create(
            name=self.rolename,
            raw_allowed_apps=self._get_raw_allowed_apps(),
        )
        self.cred = SetCredentials.objects.create(
            value=self.value,
            set_type=self.set_type,
            role=self.role,
        )
        ctype_list = self._get_ctype_create_list()
        self.role.creatable_ctypes.set(ctype_list)
        self.role.exportable_ctypes.set(ctype_list)
        return True

    def _get_raw_allowed_apps(self):
        return '\n'.join(
            [
                app.label
                for app in creme_app_configs()
                if app.credentials & CremeAppConfig.CRED_REGULAR
            ]
        )

    def _get_ctype_create_list(self):
        getct = ContentType.objects.get_for_model
        return [getct(mod).id for mod in self.create_models_list]
