import datetime
from unittest.mock import Mock, patch

from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

import pytest
from creme.creme_core.core.function_field import FunctionFieldResultsList
from creme.creme_core.models import CremeUser

from center_pilote.info_tables.models import Adhesion, TypeAdhesion
from center_pilote.pilote_extended import function_fields as ffields
from center_pilote.pilote_extended.models import Contact, Organisation

from .factories import OrganisationFactory


@pytest.fixture
def user0():
    return CremeUser.objects.create(username="user0")


@pytest.fixture
def type_adh0():
    return TypeAdhesion.objects.create(name="type_adh0")


@pytest.fixture
def contact0(user0):
    return Contact.objects.create(last_name="contact0", user=user0)


@pytest.fixture
def org0(user0, contact0):
    return Organisation.objects.create(
        name="org0", user=user0, contact_adherent=contact0
    )


@pytest.fixture
def adh1(type_adh0, org0):
    ct_id = ContentType.objects.get_for_model(org0).id
    return Adhesion.objects.create(
        type_adhesion=type_adh0,
        annee=2020,
        entity=org0,
        entity_content_type_id=ct_id,
    )


@pytest.mark.parametrize(
    'annee_cur,exp_ret',
    [
        (2020, 'oui'),
        (2022, 'non'),
    ],
)
@pytest.mark.django_db
@patch('center_pilote.pilote_extended.models.organisation.datetime')
def test_adherent_annee_field_repr(
    patched, org0, user0, adh1, annee_cur, exp_ret
):
    patched.date.today = Mock(return_value=datetime.date(annee_cur, 1, 1))
    ret = ffields.IsAdherentField()(org0, user0)
    assert str(ret) == exp_ret


@pytest.mark.parametrize(
    'annee_cur,value,exp_count',
    [
        (2020, None, 1),
        (2020, False, 0),
        (2020, True, 1),
        (2021, None, 1),
        (2021, False, 0),
        (2021, True, 1),
        (2022, None, 1),
        (2022, False, 1),
        (2022, True, 0),
    ],
)
@pytest.mark.django_db
@patch('center_pilote.pilote_extended.function_fields.datetime')
def test_adherent_annee_search_field_to_python(
    patched, org0, user0, adh1, annee_cur, value, exp_count
):
    patched.date.today = Mock(return_value=datetime.date(annee_cur, 1, 1))
    ret_q = ffields.IsAdherentSearchField.to_python(None, value)
    assert Organisation.objects.filter(ret_q).count() == exp_count


@pytest.mark.django_db
class TestCovRelIsSymbol:
    "test de couverture basic verifiant juste les types"

    @pytest.mark.parametrize('value', [None, 'john'])
    def test_cov_RelIsManagerSearchField(self, user0, value):
        class_test = ffields.RelIsManagerSearchField
        func_test = class_test.to_python
        # on a pas besoin d'instancier la classe
        # la methode to_python ne fait reference
        # qu'a l'attribut rtype_sym disponible dans
        # la classe elle meme
        ret = func_test(class_test, value)
        assert isinstance(ret, Q)

    @pytest.mark.django_db
    def test_cov_RelIsManagerField(self):
        class_test = ffields.RelIsManagerField
        func_test = class_test.__call__
        entity = OrganisationFactory()
        ret = func_test(class_test, entity, entity.user)
        assert isinstance(ret, FunctionFieldResultsList)
