import datetime
from unittest.mock import Mock, patch

from django.contrib.contenttypes.models import ContentType

import pytest
from creme.creme_core.models import CremeUser

from center_pilote.info_tables.models import Adhesion, TypeAdhesion
from center_pilote.pilote_extended.models import Contact, Organisation


@pytest.fixture
def user0():
    return CremeUser.objects.create(username="user0")


@pytest.fixture
def type_adh0():
    return TypeAdhesion.objects.create(name="type_adh0")


@pytest.fixture
def contact0(user0):
    return Contact.objects.create(last_name="contact0", user=user0)


@pytest.fixture
def adh0(type_adh0, contact0):
    ct_id = ContentType.objects.get_for_model(contact0).id
    return Adhesion.objects.create(
        type_adhesion=type_adh0,
        annee=2020,
        entity=contact0,
        entity_content_type_id=ct_id,
    )


@pytest.fixture
def org0(user0, contact0):
    return Organisation.objects.create(
        name="org0", user=user0, contact_adherent=contact0
    )


@pytest.fixture
def adh1(type_adh0, org0):
    ct_id = ContentType.objects.get_for_model(org0).id
    return Adhesion.objects.create(
        type_adhesion=type_adh0,
        annee=2020,
        entity=org0,
        entity_content_type_id=ct_id,
    )


@pytest.fixture
def org_alone(user0):
    return Organisation.objects.create(name="org_alone", user=user0)


@pytest.mark.parametrize(
    'annee_cur, exp_result',
    [
        (2019, False),
        (2020, True),
        (2021, True),
        (2022, False),
    ],
)
@pytest.mark.django_db
def test_contact_est_adherent_annee(contact0, adh0, annee_cur, exp_result):
    assert contact0.est_adherent_annee(annee_cur) is exp_result


@pytest.mark.parametrize(
    'annee_patch, exp_result, exp_str',
    [
        (2019, False, "org0"),
        (2020, True, "org0 *"),
        (2021, True, "org0 *"),
        (2022, False, "org0"),
    ],
)
@pytest.mark.django_db
@patch('center_pilote.pilote_extended.models.organisation.datetime')
def test_org_est_adherent_annee_patch(
    patched, org0, adh1, annee_patch, exp_result, exp_str
):
    patched.date.today = Mock(return_value=datetime.date(annee_patch, 1, 1))
    assert org0.est_adherent_annee() is exp_result
    assert str(org0) == exp_str


@pytest.mark.parametrize(
    'annee_patch, exp_result, exp_str',
    [
        (2019, False, "contact0"),
        (2020, True, "contact0 *"),
        (2021, True, "contact0 *"),
        (2022, False, "contact0"),
    ],
)
@pytest.mark.django_db
@patch('center_pilote.pilote_extended.models.contact.datetime')
def test_contact_est_adherent_annee_patch(
    patched, contact0, adh0, annee_patch, exp_result, exp_str
):
    patched.date.today = Mock(return_value=datetime.date(annee_patch, 1, 1))
    assert contact0.est_adherent_annee() is exp_result
    assert str(contact0) == exp_str


@pytest.mark.django_db
def test_org_est_adherent_wo_adherent(org_alone):
    assert org_alone.est_adherent_annee(2020) is False
    assert str(org_alone) == "org_alone"
