from django.utils.text import slugify

from . import constants


def rtype_sym_sanity_check(sym):
    "avoid injection by checking sym is allowed"
    assert (
        sym in constants.ALLOWED_REL_TYPE_SYM_SET
    ), f'{sym} not in ALLOWED_REL_TYPE_SYM_SET'
    return sym


def rtype_ids_from_sym(sym):
    "get rtype_ids from sym described in constants.relations"
    return getattr(constants, rtype_sym_sanity_check(sym))


def compress_interval(elm_list):
    """
    giving a sorted elm_list, this function
    try to compress the output by remplacing
    contiguous integer value with interval:

    [1,2,3,4] -> '1-4'
    [1,3,5] -> '1,3,5'
    [1,2] -> '1-2'
    [] -> ''
    [1] -> '1'
    """
    elm_iter = iter(elm_list)
    try:
        e_cur = e_start = next(elm_iter)
    except StopIteration:
        return ''
    interval = []

    def str_interval(e_start, e_cur):
        return f'{e_cur}' if e_start == e_cur else f'{e_start}-{e_cur}'

    for e_ahead in elm_iter:
        if e_cur + 1 != e_ahead:
            interval.append(str_interval(e_start, e_cur))
            e_start = e_ahead
        e_cur = e_ahead
    interval.append(str_interval(e_start, e_cur))
    return ', '.join(interval)


def join_slugify(*args):
    return '_'.join(map(lambda f: str(slugify(f)), args))


class SlugQuerySetMixin:
    """
    regroup helper methods about slug computation,
    to be mixed with a QuerySet
    """

    def refresh_slug(self):
        need_update = 0
        for cpt, elm in enumerate(self, start=1):
            if not elm.is_slug_up_to_date(update=True):
                elm.save(compute_slug=False)
                need_update += 1
        return (cpt, need_update)


class SlugModelMixin:
    """
    regroup helper methods about slug computation,
    to be mixed with a Model
    """

    def slug(self):
        """
        this method should return a string that represents
        a slug
        """
        raise NotImplementedError("Override slug() method")

    def is_slug_up_to_date(self, update=False):
        """
        return if self.slug_id reflect self.slug()
        if update=False (default) nothing is changed, method
        is idempotent.

        if update=True, self.slug_id is updated (but object
        is not saved) it allow preparation of object before
        saving it without re-calling self.slug()
        """
        computed_slug = self.slug()
        status = computed_slug == self.slug_id
        if update:
            self.slug_id = computed_slug
        return status

    def save(self, *args, **kwargs):
        """
        Compute slug_id before save (default)
        if compute_slug = False, assume that slug_id
        has been updated by another mean
        """
        compute_slug = kwargs.pop('compute_slug', True)
        if compute_slug:
            self.slug_id = self.slug()
        super().save(*args, **kwargs)
