import logging

from django.conf import settings

logger = logging.getLogger(__name__)


class Populate:
    """
    boilerplate to:
    - remove existing populated items if overwrite
    - populate if populated items does not exists
    - return True if populate occur, False otherwise

    There are 3 hooks methods:

    def remove_existing(self):
        # remove populated items

    def is_already_populated(self):
        # test if populated items exists
        return True/False

    def populate(self):
        # populate items

    Usage:

    populate_occur = InheritedPopulateClass(overwrite=..., keyword=...)()
    """

    already_populated = False
    overwrite = False

    def __init__(self, **kwargs):
        self._name = self.__class__.__name__
        self.overwrite = settings.POPULATE_OVERWRITE
        for key, value in kwargs.items():
            setattr(self, key, value)

    def _remove_existing(self):
        hook_remove_existing = getattr(self, 'remove_existing', lambda: None)
        hook_remove_existing()
        self.already_populated = False

    def _is_already_populated(self):
        hook_is_already_populated = getattr(
            self, 'is_already_populated', lambda: self.already_populated
        )
        self.already_populated = hook_is_already_populated()
        return self.already_populated

    def _populate(self):
        hook_populate = getattr(self, 'populate', lambda: None)
        hook_populate()
        self.already_populated = True

    def run(self):
        if self.overwrite:
            logger.debug(f"{self._name}: overwrite, remove existing")
            self._remove_existing()
        if self._is_already_populated():
            logger.debug(f"{self._name}: already populated, no-op")
            return False
        logger.debug(f"{self._name}: populate")
        self._populate()
        return True
