"""
Django settings for creme_center_fibois project.

For more information on this file, see
https://docs.djangoproject.com/en/stable/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/stable/ref/settings/
"""
import os.path
import sys
from email.utils import getaddresses

from creme.settings import *  # NOQA
from creme.settings import (
    CREME_OPT_CSS,
    INSTALLED_DJANGO_APPS,
    LOGGING,
    TEMPLATES,
)

from . import base_dir, env, root_dir

if sys.warnoptions:
    import warnings

    warnings.resetwarnings()
    for arg in sys.warnoptions:
        warnings._setoption(arg)

# ENVIRONMENT VARIABLES AND PATHS
# ------------------------------------------------------------------------------

# Local directory used for static and templates overrides
local_dir = base_dir.path('local')

# Directory for variable stuffs, i.e. user-uploaded media
var_dir = base_dir.path('var')
if not os.path.isdir(var_dir()):
    os.mkdir(var_dir(), mode=0o755)

# Location on which the application is served
APP_LOCATION = env('APP_LOCATION', default='/')

LOG_LEVEL = env('LOG_LEVEL', default='WARNING').upper()

LOGGING['handlers']['console']['level'] = LOG_LEVEL
LOGGING['loggers']['']['level'] = LOG_LEVEL

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#debug
DEBUG = env.bool('DJANGO_DEBUG', default=True)

# Local time zone for this installation
TIME_ZONE = 'Europe/Paris'

# https://docs.djangoproject.com/en/stable/ref/settings/#language-code
LANGUAGE_CODE = 'fr'

# https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1

# https://docs.djangoproject.com/en/stable/ref/settings/#use-i18n
USE_I18N = True
# https://docs.djangoproject.com/en/stable/ref/settings/#use-l10n
USE_L10N = False
# https://docs.djangoproject.com/en/stable/ref/settings/#use-tz
USE_TZ = True

# DATABASES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#databases
# https://django-environ.readthedocs.io/en/stable/#supported-types
DATABASES = {
    'default': env.db(
        'DJANGO_DATABASE_URL',
        default='sqlite:///{}'.format(base_dir('sqlite.db')),
    ),
}

# URLS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#root-urlconf
# ROOT_URLCONF already defined by creme
ROOT_URLCONF = 'center_pilote.urls'
# https://docs.djangoproject.com/en/stable/ref/settings/#wsgi-application
WSGI_APPLICATION = 'center_pilote.wsgi.application'

# APP CONFIGURATION
# ------------------------------------------------------------------------------
# INSTALLED_DJANGO_APPS already defined by creme

# Project applications
LOCAL_APPS = [
    'django_fieldbustier',
    'pg_copy',
    'center_pilote.pilote_extended',
]

INSTALLED_CREME_APPS = [
    # CREME CORE APPS
    'creme.creme_core',
    'creme.creme_config',
    'creme.documents',
    'creme.activities',  # Extra features if 'assistants' is installed.
    'creme.persons',
    # CREME OPTIONAL APPS (can be safely commented)
    'creme.assistants',
    'creme.graphs',
    'creme.reports',
    'creme.products',
    # 'creme.recurrents',
    # 'creme.billing',  # Need 'products'
    'creme.opportunities',  # Need 'products'. Extra features with 'billing'
    # 'creme.commercial',  # Need 'opportunities'
    'creme.events',  # Need 'opportunities'
    'creme.crudity',
    'creme.emails',  # Extra features if 'crudity' is installed.
    # 'creme.sms',  # Work In Progress
    'creme.projects',
    'creme.tickets',
    # 'creme.cti',
    'creme.vcfs',
    # 'creme.polls',  # Need 'commercial'
    # 'creme.mobile',
    'creme.geolocation',
    'center_pilote.info_tables',
    'center_pilote.filiere_bois',
    'center_pilote.bleach',
]

# https://docs.djangoproject.com/en/stable/ref/settings/#installed-apps
INSTALLED_APPS = INSTALLED_DJANGO_APPS + LOCAL_APPS + INSTALLED_CREME_APPS


# PASSWORDS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#password-hashers
PASSWORD_HASHERS = [
    # https://docs.djangoproject.com/en/stable/topics/auth/passwords/#using-argon2-with-django
    # 'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
]

# https://docs.djangoproject.com/en/stable/topics/auth/passwords/#password-validation
# AUTH_PASSWORD_VALIDATORS # already defined by creme

# MIDDLEWARE
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#middleware
# MIDDLEWARE # already defined by creme

# STATIC
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#static-files
STATIC_ROOT = var_dir('static')

# https://docs.djangoproject.com/en/stable/ref/settings/#static-url
STATIC_URL = os.path.join(APP_LOCATION, 'static_media/')

# https://docs.djangoproject.com/en/stable/ref/settings/#staticfiles-dirs
STATICFILES_DIRS = [root_dir('static')]
if os.path.isdir(local_dir('static')):
    STATICFILES_DIRS.insert(0, local_dir('static'))

# https://docs.djangoproject.com/en/stable/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

# MEDIA
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#media-root
# MEDIA_ROOT = var_dir('media')  # already defined by creme

# https://docs.djangoproject.com/en/stable/ref/settings/#media-url
# MEDIA_URL = os.path.join(APP_LOCATION, 'media/')  # already defined by creme

# Optional js/css bundles for extending projects.
# Beware to clashes with existing bundles ('main.js', 'l10n.js').

CREME_OPT_CSS += [
    (
        'center_pilote.pilote_extended',
        'pilote_extended/css/pilote_extended.css',
    ),
]
GENERATED_MEDIA_DIR = var_dir('static/')

# https://docs.djangoproject.com/en/stable/ref/settings/#file-upload-directory-permissions
FILE_UPLOAD_DIRECTORY_PERMISSIONS = 0o755
# https://docs.djangoproject.com/en/stable/ref/settings/#file-upload-permissions
FILE_UPLOAD_PERMISSIONS = 0o644

# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#templates
# TEMPLATES  # already defined by creme
TEMPLATES[0]['DIRS'].insert(0, root_dir('customer_templates'))
if os.path.isdir(local_dir('templates')):
    TEMPLATES[0]['DIRS'].insert(0, local_dir('templates'))

# patch get_version to add center_pilote sub version
TEMPLATES[0]['OPTIONS']['context_processors'] = [
    'center_pilote.context_processors.get_version'
    if elm == 'creme.creme_core.context_processors.get_version'
    else elm
    for elm in TEMPLATES[0]['OPTIONS']['context_processors']
]

# FIXTURES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#fixture-dirs
FIXTURE_DIRS = [root_dir('fixtures')]

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/topics/email/#email-backends
# https://django-environ.readthedocs.io/en/stable/#supported-types
vars().update(env.email_url('DJANGO_EMAIL_URL', default='smtp://localhost:25'))

DEFAULT_FROM_EMAIL = env('DEFAULT_FROM_EMAIL', default='webmaster@localhost')
# Use the same email address for error messages
SERVER_EMAIL = DEFAULT_FROM_EMAIL

# ADMIN
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#admins
ADMINS = getaddresses([env('ADMINS', default='Cliss XXI <tech@cliss21.com>')])

# https://docs.djangoproject.com/en/stable/ref/settings/#managers
MANAGERS = ADMINS

# SESSIONS AND COOKIES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/stable/ref/settings/#session-cookie-path
SESSION_COOKIE_PATH = APP_LOCATION

# https://docs.djangoproject.com/en/stable/ref/settings/#csrf-cookie-path
CSRF_COOKIE_PATH = APP_LOCATION

# FIELDBUSTIER
# ------------------------------------------------------------------------------
# https://github.com/mrjmad/django-fieldbustier
# Set to True if you want fieldbustier to generate migrations
GENERATE_FIELDBUSTIER_MIGRATIONS = False
# where to store fieldbustier migrations, remember to
# 1. create python module (create directory and __init__.py within)
# 2. copy migrations from original django application
# 3. hack model with fieldbustier then ./manage makemigrations ...
# copy corresponding migrations from original
# MIGRATION_MODULES = {'persons': 'center_pilote.creme_migrations.persons'}

ADD_FIELD_DJANGO_FIELDBUSTIER = []
REPLACE_FIELD_DJANGO_FIELDBUSTIER = []
DELETE_FIELD_DJANGO_FIELDBUSTIER = []


# ------------------------------------------------------------------------------
# APPLICATION AND 3RD PARTY LIBRARY SETTINGS
# ------------------------------------------------------------------------------
PG_COPY_BACKUP_PATH = var_dir('backups')

PERSONS_CONTACT_MODEL = 'pilote_extended.Contact'
PERSONS_CONTACT_FORCE_NOT_CUSTOM = True

PERSONS_ORGANISATION_MODEL = 'pilote_extended.Organisation'
PERSONS_ORGANISATION_FORCE_NOT_CUSTOM = True

PERSONS_ADDRESS_MODEL = 'pilote_extended.Address'
PERSONS_ADDRESS_FORCE_NOT_CUSTOM = True

ACTIVITIES_ACTIVITY_MODEL = 'pilote_extended.Activity'
ACTIVITIES_ACTIVITY_FORCE_NOT_CUSTOM = True

POPULATE_OVERWRITE = env.bool('POPULATE_OVERWRITE', default=False)
TOKEN_BOIS_ET_VOUS = env('TOKEN_BOIS_ET_VOUS', default=None)
